/*
 *  Copyright (c) 2006-2008, Lars Engels <lars.engels@0x20.net>
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without modification,
 *  are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation and/or
 *     other materials provided with the distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <xosd.h>

#include <sys/socket.h>
#include <sys/sysctl.h>
#include <sys/types.h>
#include <sys/un.h>

#include "nbosd.h"

int
main(int argc, char *argv[]) {
  char *p;
  int ch;

  SYSCTL_INFO sysctl_info;
  CONFIG config;

  /* set initial default config */
  config.font  =     FONT;
  config.color =     COLOR;
  config.pos_x =     POS_X;
  config.pos_y =     POS_Y;
  config.width =     XOSD_WIDTH;
  config.sleeptime = SLEEP;
  config.alarm =     ALARM_PCT;
  config.daemon =    0;

  while ((ch = getopt(argc, argv, "a:c:f:dhx:w:y:t:")) != -1)
    switch (ch) {
    case 'a':
      config.alarm = atoi(optarg);
      break;
    case 'c':
      config.color = optarg;
      break;
    case 'd':
      config.daemon = 1;
      break;
    case 'f':
      config.font = optarg;
      break;
    case 't':
      if (config.sleeptime != 0) config.sleeptime = atoi(optarg) * 1000;
      break;
    case 'w':
      config.width = atoi(optarg);
      break;
    case 'x':
      config.pos_x = strtol(optarg, &p, 10);
      break;
    case 'y':
      config.pos_y = strtol(optarg, &p, 10);
      break;
    case '?':
    case 'h':
    default:
      usage();
      break;
    }
  argc -= optind;
  argv += optind;

  if (config.daemon)
    devd_connect(&config);
  else {
      get_sysctls(&sysctl_info);
      show_bars(&sysctl_info, &config);
  }

  exit(0);
}

/* get available sysctls */
static void
get_sysctls(SYSCTL_INFO *sysctl_info) {
  sysctl_info->len_int = sizeof(int);
  sysctl_info->len_freq = sizeof(sysctl_info->freq_levels);
  sysctl_info->cpu_freq_avail = sysctlbyname(SYSCTL_CPU_FREQ,
      &sysctl_info->cpu_freq, &sysctl_info->len_int, NULL, 0);
  sysctl_info->freq_levels_avail = sysctlbyname(SYSCTL_CPU_FREQ_LEVELS,
      sysctl_info->freq_levels, &sysctl_info->len_freq, NULL, 0);
  sysctl_info->battery_life_avail = sysctlbyname(SYSCTL_BATTERY_LIFE,
      &sysctl_info->battery_life, &sysctl_info->len_int, NULL, 0);
  sysctl_info->battery_state_avail = sysctlbyname(SYSCTL_BATTERY_STATE,
      &sysctl_info->battery_state, &sysctl_info->len_int, NULL, 0);
  sysctl_info->battery_time_avail = sysctlbyname(SYSCTL_BATTERY_TIME,
      &sysctl_info->battery_time, &sysctl_info->len_int, NULL, 0);
}

static void
show_bars(SYSCTL_INFO *sysctl_info, CONFIG *config) {
  char *cpu_title     = "CPU Frequency: ";
  char *battery_title = "Battery Life: ";
  char *state_prefix  = "AC state: ";
  char *time_prefix   = "Time left: ";
  char state[100];
  int frequency;

  osd = xosd_create(5); /* create a four lined xosd */

  if (osd == NULL) {
    perror ("Could not create \"osd\"");
    exit(1);
  }

  /* set how the osd should look like */
  xosd_set_font (osd, config->font);
  if (sysctl_info->battery_life <= config->alarm)
    xosd_set_colour (osd, ALARM_COLOR);
  else
    xosd_set_colour (osd, config->color);
  xosd_set_outline_offset (osd, 1);
  xosd_set_shadow_offset (osd, 4);
  xosd_set_outline_colour (osd, "black");
  xosd_set_align (osd, XOSD_left);
  xosd_set_pos (osd, XOSD_bottom);
  xosd_set_horizontal_offset (osd, config->pos_x);
  xosd_set_vertical_offset (osd, config->pos_y);
  xosd_set_bar_length(osd, config->width);

  /* show the OSD */
  if (sysctl_info->freq_levels_avail >= 0 && sysctl_info->cpu_freq_avail >= 0) {
    /* display CPU frequency */
    xosd_display(osd, 0, XOSD_printf, "%s  %i MHz",cpu_title,
        sysctl_info->cpu_freq);
    frequency = sysctl_info->cpu_freq / (strtoq(sysctl_info->freq_levels,
          NULL, 10) / 100);
    xosd_display(osd, 1, XOSD_percentage, sysctl_info->cpu_freq
        / (strtoq(sysctl_info->freq_levels, NULL, 10) / 100));
  }

  /* did we find a battery? if yes, show its capacity  value and bar */
  if (sysctl_info->battery_life_avail >= 0 && sysctl_info->battery_life >= 0 &&
                  sysctl_info->battery_time_avail >= 0) {
    if (sysctl_info->battery_time >= 0) { /* with remaining battery time */
      xosd_display(osd, 2, XOSD_printf, "%s  %i %% (%d mins)", battery_title,
                      sysctl_info->battery_life, sysctl_info->battery_time);
    }
    else /* without time */
      xosd_display(osd, 2, XOSD_printf, "%s  %i %%", battery_title, sysctl_info->battery_life);

    xosd_display(osd, 3, XOSD_percentage, sysctl_info->battery_life);
  }
  else
    xosd_display (osd, 2, XOSD_printf, "No Battery");

  /* set battery message */
  if (sysctl_info->battery_state_avail >= 0) {
    switch (sysctl_info->battery_state) {
    case BATT_FULL:
      strlcpy(state, BATT_FULL_MSG, sizeof(state));
      break;
    case BATT_DISCHARGING:
      strlcpy(state, BATT_DISCHARGING_MSG, sizeof(state));
      break;
    case BATT_CHARGING:
      strlcpy(state, BATT_CHARGING_MSG, sizeof(state));
      break;
    case BATT_CRITICAL:
      xosd_set_colour (osd, ALARM_COLOR); /* set the alarm color */
      strlcpy(state, BATT_CRITICAL_MSG, sizeof(state));
      break;
    case BATT_NONE:
      strlcpy(state, BATT_NONE_MSG, sizeof(state));
      break;
    default:  /* should not be reached */
      strlcpy(state, BATT_UNKNOWN_MSG, sizeof(state));
      break;
    }

    /* display AC state */
    if (sysctl_info->battery_time_avail > 0)
        xosd_display(osd, 4, XOSD_printf, "%s %s   %s %i", state_prefix, state, \
            time_prefix, sysctl_info->battery_time);
    else
        xosd_display(osd, 4, XOSD_printf, "%s %s", state_prefix, state);
  }

  /* nothing usable found */
  if (sysctl_info->freq_levels_avail != 0 && sysctl_info->cpu_freq_avail != 0 &&
    sysctl_info->battery_life_avail != 0)
    xosd_display(osd, 0, XOSD_printf, NOTHING_FOUND_MSG);

  usleep(config->sleeptime);
  xosd_destroy(osd);
}

static void
devd_connect (CONFIG *config) {
  int event_fd;
  struct sockaddr_un addr;
  FILE *sockin;
  char sys[100];
  char subsys[100];
  char type[100];
  char notify[100];

  SYSCTL_INFO sysctl_info;

  event_fd = socket(PF_UNIX, SOCK_STREAM, 0);
  if (event_fd < 0) {
    printf("failed to create socket: %d\n", errno);
    exit (1);
  }

  addr.sun_family = AF_UNIX;
  strncpy(addr.sun_path, SOCKET_PATH, sizeof(addr.sun_path));
  if (connect(event_fd, (struct sockaddr *)&addr, sizeof(addr)) == 0) {
    sockin = fdopen(event_fd, "r");
    while (!feof(sockin)) {
        if (fscanf(sockin, "%s %s %s %s", sys, subsys, type, notify) != 4)
          break;
        if (strncmp(subsys, BATTERY_EVENT, sizeof(subsys))) {
          get_sysctls(&sysctl_info);
          if (sysctl_info.battery_life < config->alarm &&
                          sysctl_info.battery_state != BATT_CHARGING ) {
            show_bars(&sysctl_info, config);
          }
        }
    }
    (void) fclose(sockin);
    (void) close(event_fd);
  }
  else {
    fprintf(stderr, "Could not connect. Is devd not running?\n");
    exit (1);
  }
}

static void
usage(void) {
  fprintf(stderr,
    "Usage: nbosd [-h] [-a alarm percentage] [-c color] [-d] [-f font]"
    "[-t time (ms)] [-w bar width] [-x x-offset] [-y y-offset]\n");
  exit (1);
}

/* vim:set ts=2 expandtab: */
